UOregon's Phase 3 Granular System Manual

# System Description
Our system for extracting granular events is a pipeline that consists of the following models running in order:
- Trigger Model: finds spans and types for basic event triggers. The model operates as follows:
    + XLM-Roberta is used to encode words in a sentence.
    + Word representations are sent to a Conditional Random Field layer to find spans and types for basic event triggers.
- Argument Model: finds spans and roles for arguments for detected basic event triggers via sequence labeling. The model operates as follows:
    + XLM-Roberta is used to encode words in a sentence. We also compute word features: parts of speech, distance to a given trigger, dependency relations.
    + Word representations are sent to a Conditional Random Field layer to find spans and types for entity mentions.
    + Trigger representations are paired with entity representations to obtain argument roles for each entity mention.
- Template Model: finds template types indicated by a given document.
    + XLM-Roberta is used to encode words in a given document. Spans and types for basic event triggers in each sentence of the document are also used to compute word features.
    + [CLS] representation of the document along with pooled word features are used as final representation for the document.
    + The final representation for the document is sent to different binary classifiers to detect templates of different types exhibited by the document. Each detected template type represents a template for the document.
- Slot Model: finds spans and slot types for slot fillers for detected templates.
    + XLM-Roberta is used to encode words in each sentence of a given document.
    + Basic event triggers and arguments are considered as slot filler candidates for a given template. Representation for each filler candidate is computed as average sum of representations for words inside the candidate span.
    + Representations for each slot filler candidate are sent to different classifiers to identify slot types and features for each filler candidate.
    + Slot predictions are filtered by confidence scores to retain the most confident slot fillers at test time.
- Joint Event Reference and Coreference Resolution Model: finds coreferred basic event triggers to deduplicate basic events and granular slot fillers.
    + XLM-Roberta is used to encode words in a sentence. Spans and types for basic event triggers in the sentence are also used to compute word features.
    + We consider all possible pairs among basic event triggers and classify each pair to see if given two basic triggers corefer/refer to each other or not.
    + Predictions are used to form basic event clusters, which are finally utilized to find ref-events for each basic event, deduplicate basic events and slot fillers.

Note that, we also employ a weighting mechanism for training losses computed based on Phase 1 data during the training of the models.

# Runing the System
Please follow the steps below to run our system:
## Preparation
1. Create a conda environment using `./granular.yml`.
2. Make sure all the data files defined in `./constants.py` are available at their designated locations. For your convenience, we already included all the data files inside this code.

## Running
1. Create fold-based training data and generate a vocabulary:
```
python cross_valid.py $fold; python util.py 0
```
where $fold is the fold identity, ranging from 0 to 9.

2. Prepare data for the trigger model:
```
python util.py 1
```
3. Train the trigger model:
```
python train.py --model_name trg --xlmr_model_name xlm-roberta-large --batch_size 16 --xlmr_learning_rate 2e-5 --event_hidden_num 450 --accumulate_step 4 --max_epoch 50 --p1_data_weight 0.1 --learning_rate 6e-4
```
4. Prepare data for the argument model (predictions from the trigger model will be used in this process):
```
python util.py 2
```
5. Train the argument model:
```
python train.py --model_name arg --xlmr_model_name xlm-roberta-large --batch_size 16 --xlmr_learning_rate 2e-5 --max_epoch 80 --p1_data_weight 0.4 --event_hidden_num 450 --role_hidden_num 350 --learning_rate 2e-4
```
6. Prepare data for the template model (predictions from the trigger model will be used in this process):
```
python util.py 3
```
7. Train the template model:
```
python train.py --model_name template --xlmr_model_name xlm-roberta-large --batch_size 10 --xlmr_learning_rate 2e-5 --event_hidden_num 450 --accumulate_step 4 --max_epoch 50 --p1_data_weight 0.1 --learning_rate 2e-4
```
8. Prepare data for the slot model (predictions from the trigger, argument, and template models will be used in this process):
```
python util.py 4
```
9. Train the slot model:
```
python train.py --model_name slot --xlmr_model_name xlm-roberta-large --batch_size 16 --xlmr_learning_rate 2e-5 --max_epoch 80 --event_hidden_num 450 --role_hidden_num 350 --p1_data_weight 0.1 --learning_rate 1e-3
```
10. Prepare data for the joint event reference and coreference resolution model (predictions from the trigger model will be used in this process):
```
python util.py 5
```
11. Train the joint event reference and coreference resolution model:
```
python train.py --model_name coref --xlmr_model_name xlm-roberta-large --batch_size 16 --xlmr_learning_rate 2e-5 --accumulate_step 4 --max_epoch 50 --event_hidden_num 500 --p1_data_weight 0.2 --learning_rate 9e-4
```
12. Once all the training is done, granular event extraction can be done over `./app/test_data.bp.json` via running the command:
```
python train.py --model_name pipeline
```
The system should split the test data into different parts, each corresponds to one of the target languages (Chinese, Korean, Russian) to make predictions for each language using their respective model checkpoints. In the end, all predictions are combined and written to `./app/results.json`.
