# BETTER Program Library

The BETTER Program Library (`bp_lib`) is a Python library for wrangling JSON data in the BETTER
Program schema ("BP JSON") and scoring BP JSON documents against each other.

[[_TOC_]]

## Development Rules

This project was developed under the following rules and guidelines:

- No binary blobs.
- No data storage (config files are okay). This means:
    - No MAT documents
    - No BP documents
    - No log files or `stdout` outputs captures
    - This rule _does not_ apply to testing fixtures.
- Scripts should be executable, with `bash` shebangs. See
  [the Python script template](lib/__script_template.py) for an example. Library files should
  not have shebangs and not be executable.
- Scripts should **_not_** live in `lib/`. Only library code should live in `lib/`. Scripts should
  live in folders parent to `lib/`.
- Never push to `master`. Always create a merge request to `master` from your feature branches.
- Never develop in `master`. Create a new branch for a new feature, preferably with an Issue if it's
  sufficiently large in scope, but it doesn't have to reference an Issue.
    - Branches should have descriptive names. No version numbers, just short and simple names.
    - If your new branch is solely in response to an Issue, since Issues for this group can appear
      in multiple repositories, use the naming convention `repo_name#issue_number` instead of a
      name if the Issue is logged in another repository, or `Issue#issue_number` for Issues logged
      in _this_ repositiory.
- Always release from `master`, and tag releases.
- Write appropriate `pytest` cases in the [`tests` folder](tests). Include data files used for
  testing in [`tests/test_data`](tests/test_data).
- Follow the [Google Python Style Guide](http://google.github.io/styleguide/pyguide.html).

## Python Environment Setup

You'll need at least Python 3.9. This project was primarily developed on Ubuntu 20.04, with a
Python virtual environment set up like so (you don't need to use a virtual environment, but one
is **highly recommended**):

```bash
cd /path/to/bp_lib
python -m venv venv --prompt="bp_lib"
source venv/bin/activate # to enter the environment; use `deactivate` to leave the environment
```

Python, via the `venv` module, copies that Python installation when creating the virtual
environment to a new location. The version installed on your system (if one is installed at all),
may or may not be sufficient. You'll need Python 3.9 or higher for the scripts to work. If you don't
have Python 3 installed:

- One is usually available the Linux package manager (i.e., `apt` on Ubuntu)
- One is available in `homebrew` for MacOS
- You could use the system package manager in
  [WSL on Windows 10/11](https://docs.microsoft.com/en-us/windows/wsl/install)
- You can install it from source yourself (via https://python.org)
- You can use a language version manager like [`asdf`](https://asdf-vm.com)

The project has been well-tested with versions as new as Python 3.10. Note that it *has not* been
tested on Windows native; it has only seen use in Linux or Unix environments, including
[WSL](https://docs.microsoft.com/en-us/windows/wsl/about).

Regardless of whether you're using a virtual environment, you'll need to install the dependencies
like so:

```bash
python -m pip install -r requirements.txt --no-deps
```

Ever since `pip 20.0` was release, the new dependency resolver causes ridiculous amounts of
slowness. The dependencies listed `requirements.txt` have been verified to work together as they are
currently listed. We skip the dependency resolution step with the `--no-deps` option.

## Docs

The project ships with a `sphinx` documentation store in the [`docs/`](docs) folder. It is best
viewed in its HTML form. Building this HTML is easy:

```bash
cd docs
make html
```

The documentation can be viewed by opening `docs/build/html/index.html` in any web browser.

## Testing

All of the test cases live in [`tests/`](tests). To run them, call `pytest` from the root of the
package.

# NOTICE

This software (or technical data) was produced for the U.S. Government under contract
2015-14120200002-002, and is subject to the Rights in Data-General Clause 52.227-14 , ALT IV (MAY
2014) or (DEC 2007).

© 2022 The MITRE Corporation. All Rights Reserved.
