# Copyright (C) 2007 - 2021 The MITRE Corporation. See the toplevel
# file LICENSE.txt for license terms.


# This is the core Python library for the MITRE Annotation Toolkit.

from setuptools import setup

import os
thisDir = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(thisDir, "MAT", "MAT_VERSION")) as fp:
    matVersion = fp.read().strip()

# Note, as far as I can tell, that in order to ensure that sdist
# and install both include the relevant extra files, you have to
# have both a MANIFEST.in and the following settings in setup().
# At least, a comment here:
# https://stackoverflow.com/questions/36335720/including-data-files-in-python-build-and-install
# suggests that this is necessary, and certainly, without
# MANIFEST.in the files weren't included in the sdist, and
# without package_data they weren't installed. I'd hoped that
# Python package management wasn't nearly as badly documented
# and idiotic as Java package management, but apparently I was
# wrong.
    
setup(
    name="MAT",
    version=matVersion,
    packages=["MAT"],
    url="https://gitlab.mitre.org/MITRE_Annotation_Toolkit/MAT",
    description="The MITRE Annotation Toolkit Python API",
    maintainer="Sam Bayer",
    maintainer_email="sam@mitre.org",
    install_requires=["munkres==1.1.2"],
    package_data = {"MAT": ["MAT/task_template.xml",
                            "MAT/asd/template.xml",
                            "MAT/doc_convertor_template.xml",
                            "MAT/ws_db.sql",
                            "MAT/MAT_VERSION"]},
    include_package_data=True                        
)
